<?php

/**
 * @file
 * Template functions for bilderblock theme.
 */

/**
 * Currently non-configurable style values.
 */
// Border width for expanded (hovered) slide side stripes.
// Used for calculation of inner content width.
// @see includes/dynamic_styles.inc
define('BILDERBLOCK_SLIDE_SIDE_CONTENT_BORDER', 1);

/**
 * Implements hook_preprocess_html().
 */
function bilderblock_preprocess_html(&$vars) {

  $theme_path = drupal_get_path('theme', 'bilderblock');

  // Dynamic CSS is in public files folder.
  if ($dynamic_css = bilderblock_dynamic_css()) {
    drupal_add_css($dynamic_css, array(
      'type' => 'inline',
      'group' => CSS_THEME,
    ));
  }

  // Prepare inline CSS options.
  $options = array(
    'type' => 'inline',
    'group' => CSS_THEME,
    'every_page' => TRUE,
  );
  // Add logo and menu icon settings.
  foreach (array(
    'logo' => "div#menu-logo > a:before {content: url('!icon');}",
    'menu_icon' => "a#menu-icon:before {content: url('!icon');}",
  ) as $stub => $css) {
    if ($icon = bilderblock_logo_path($stub)) {
      drupal_add_css(format_string($css, array('!icon' => $icon)), $options);
    }
  }
  // Add variables for site_name and site_slogan.
  drupal_add_css(
    format_string("div#menu-title div.name > a:before {content: '!site_name';} div#menu-title div.slogan:before {content: '!site_slogan';}", array(
      '!site_name' => variable_get('site_name'),
      '!site_slogan' => variable_get('site_slogan'),
    )),
    $options
  );

  // Add jQuery cookie.
  drupal_add_library('system', 'jquery.cookie', TRUE);
  // Set mobile-friendly viewport.
  drupal_add_html_head(array(
    '#type' => 'html_tag',
    '#tag' => 'meta',
    '#attributes' => array(
      'name' => 'viewport',
      'content' => 'width=device-width, initial-scale=1, user-scalable=yes',
    ),
  ), 'bilderblock_viewport');

  // Add fx classes as of theme settings.
  // - Saturation, unless 100% (=ineffective)
  if (theme_get_setting('fx_saturation', 'bilderblock') != 100) {
    $vars['classes_array'][] = 'bilderblock-fx-saturation';
  }
  // - Opacity, unless 100% (=ineffective)
  if (theme_get_setting('fx_opacity', 'bilderblock') < 100) {
    $vars['classes_array'][] = 'bilderblock-fx-opacity';
  }
  // - Tint, if any.
  if (theme_get_setting('fx_tint', 'bilderblock') > 0) {
    $vars['classes_array'][] = 'bilderblock-fx-tint';
  }
  // - Clear hover effects, if active.
  if (theme_get_setting('fx_hover_clear', 'bilderblock') > 0) {
    $vars['classes_array'][] = 'bilderblock-fx-hover-clear';
  }
  // Conditionally suppress side-strip menu icon.
  if (theme_get_setting('toggle_logo', 'bilderblock')) {
    $vars['classes_array'][] = 'bilderblock-logo';
  }
  if (theme_get_setting('toggle_name', 'bilderblock')) {
    $vars['classes_array'][] = 'bilderblock-name';
  }
  if (theme_get_setting('toggle_slogan', 'bilderblock')) {
    $vars['classes_array'][] = 'bilderblock-slogan';
  }
  if (theme_get_setting('toggle_post_image_border', 'bilderblock')) {
    $vars['classes_array'][] = 'bilderblock-post-image-border';
  }


  // Evaluate fallback menu toggle links and current user menu state.

  // Determine the next menu state from theme settings (or override
  // through the client cookie, if available).
  $menu_closed =
    (
      empty($_COOKIE['Drupal_visitor_bilderblockMenu'])
      &&
      (bool) theme_get_setting('menu_default_closed', 'bilderblock')
    )
    ||
    $_COOKIE['Drupal_visitor_bilderblockMenu'] == 'closed'
  ;
  // Since themes do not integrate with menu hooks, we need to evaluate
  // a GET query parameter directly.
  if (!empty($_GET['menu']) && $_GET['menu'] == 'toggle') {
    // If we received a request to toggle the menu state,
    // this means altering the current (or setting a whole new)
    // cookie.
    $menu_closed = !$menu_closed;

    // Set/update the user cookie.
    user_cookie_save(array(
      'bilderblockMenu' => $menu_closed ? 'closed' : 'open',
    ));
    // Redirect to get rid of query string.

    drupal_goto(current_path());
  }
  // Set CSS classes according to updated menu state.
  if ($menu_closed) {
    $vars['classes_array'][] = 'bilderblock-menu-closed';
  }
}

/**
 * Implements hook_process_html().
 */
function bilderblock_process_html(&$vars) {
  // Hook into color.module.
  if (module_exists('color')) {
    _color_html_alter($vars);
  }
}

/**
 * Implements hook_preprocess_maintenance_page().
 */
function bilderblock_preprocess_maintenance_page(&$variables) {
  // By default, site_name is set to Drupal if no db connection is available
  // or during site installation. Setting site_name to an empty string makes
  // the site and update pages look cleaner.
  // @see template_preprocess_maintenance_page
  if (!$variables['db_is_active']) {
    $variables['site_name'] = '';
  }
  // @todo review drupal_add_css(drupal_get_path('theme', 'bilderblock') . '/css/maintenance-page.css');
}

/**
 * Checks if this result has been flagged as "teaser".
 *
 * @see bilderblock_preprocess_page().
 */
function bilderblock_preprocess_search_result(&$vars) {
  // In bilderblock_preprocess_page, we conditionally insert flags if we have
  // moved teasers to the "light box" (teaser node) area.
  // The following allows the template to suppress output.
  // @see search-result.tpl.php.
  $vars['is_teaser'] = !empty($vars['result']['is_teaser']);
}

/**
 * Checks if there are any results being rendered as teasers.
 *
 * Prevent "no results" section, if we have successfully taken out teasers.
 * This is safer than to simply check for $vars['search_result'] being empty;
 * also the results template should be empowered to distinguish between
 * "empty but teasers" and "no result".
 * $has_teasers can be evaluated in the template file.
 *
 * @see search-results.tpl.php
 * @see bilderblock_preprocess_search_result().
 */
function bilderblock_preprocess_search_results(&$vars) {
  foreach ((array) $vars['results'] as $item) {
    if ($item['is_teaser']) {
      $vars['has_teasers'] = TRUE;
      break;
    }
  }
}

/**
 * Re-orders page content (teasers) at theming level.
 *
 * Since the "light box"-style teaser area is theme-specific,
 * there is no Drupal way to re-order components in the page render array
 * other than to re-assemble the pre-themed render array from top to bottom.
 * This function primarily looks up whether the following (optional) components
 * exist in the system_main block (Drupal's hard-wired main content area):
 * - "nodes" (any node children in the system_main_block render array.
 * - "search_results" (result snippets from Drupal standard/core search)
 * If such elements are found, they are "moved" from their original location
 * to the 'teasers' variable, so they can be rendered in place (i. e. in the page
 * template).
 * Also note the companion workaround for blocks.
 * @see page.tpl.php
 * @see search_result.tpl.php
 * @see search_results.tpl.php
 * @see bilderblock_preprocess_block().
 */
function bilderblock_preprocess_page(&$vars) {
  // Move pager to the absolute page bottom.
  if ($vars['pager'] =@ $vars['page']['content']['system_main']['pager']) {
    unset($vars['page']['content']['system_main']['pager']);
  }

  // Prepare teaser container.
  $teasers = array();
  if (!isset($vars['teasers'])) {
    $vars['teasers'] = array();
  }
  $teasers += $vars['teasers'];

  // Extract all teaser nodes from the original array.
  if (
    isset($vars['page']['content']['system_main']['nodes'])
    &&
    is_array($vars['page']['content']['system_main']['nodes'])
    &&
    !empty($vars['page']['content']['system_main']['nodes'])
  ) {
    foreach (element_children($vars['page']['content']['system_main']['nodes']) as $nid) {
      if (
        isset($vars['page']['content']['system_main']['nodes'][$nid]['#view_mode'])
        &&
        $vars['page']['content']['system_main']['nodes'][$nid]['#view_mode'] == 'teaser'
      ) {
        $teasers[$nid] = $vars['page']['content']['system_main']['nodes'][$nid];
        unset($vars['page']['content']['system_main']['nodes'][$nid]);
      }
    }
    // Clean up nodes container, if empty.
    if (!element_children($vars['page']['content']['system_main']['nodes'])) {
      unset($vars['page']['content']['system_main']['nodes']);
    }

    // Remove term heading if there is no content.
    // In theming, there is no (known to me) option to see if the term is really
    // free from any fields without adding much overhead.
    // This workaround thus checks the complete term.
    if ($test =@ $vars['page']['content']['system_main']['term_heading']['term']) {
      // theme_taxonomy_term will always add '#prefix' and '#suffix'.
      // Thus, we will only theme all child fields.
      unset($test['#theme']);
      if (!$test = render($test)) {
        // Unless this logic is flawed, the section is empty. Drop it.
        unset($vars['page']['content']['system_main']['term_heading']);
      }
    }
    // Finally remove system main block, if there are no other children.
    if (!element_children($vars['page']['content']['system_main'])) {
      unset($vars['page']['content']['system_main']);
    }
  }
  $vars['teasers'] += $teasers;

  // Transform search results into teaser nodes, too.
  $teasers = array();
  $results =& $vars['page']['content']['system_main']['search_results']['#results'];
  // Init:
  $vars['page']['content']['system_main']['search_results']['teaser_count'] = 0;
  if (isset($results) && is_array($results) && !empty($results)) {
    foreach (element_children($results) as $nid) {
      if (
        (isset($results[$nid]['node']))
        &&
        ($teaser = node_view($results[$nid]['node'], 'teaser'))
      ) {
        $teasers[$nid] = $teaser;
        $results[$nid]['is_teaser'] = 1;
      }
    }
  }
  $vars['teasers'] += $teasers;

  // Set slide/exposed image.
  $slide = array();
  if (
    (!empty($vars['node']))
    &&
    ($nodes =@ $vars['page']['content']['system_main']['nodes'])
    &&
    ($nid = element_children($nodes))
  ) {
    if ($nid = array_shift($nid)) {
      if ($imgfield = _bilderblock_image_source($nodes[$nid], $nid)) {
        $vars['slide'] = $imgfield;
        unset($vars['page']['content']['system_main']['nodes'][$nid][$imgfield['#field_name']]);
      }
    }
  }

  // Provide a menu toggle link.
  $vars['menu_toggle_link'] = url(current_path(), array(
    'query' => array(
      'menu' => 'toggle',
    ),
  ));
}

/**
 * Override or insert variables into the page template.
 */
function bilderblock_process_page(&$vars) {
  // Set toggle_breadcrumb var.
  if (!theme_get_setting('toggle_breadcrumb', 'bilderblock')) {
    $vars['breadcrumb'] = '';
  }
}

function bilderblock_preprocess_block(&$vars) {
  // Workaround. Motivation:
  // Determine whether inside a block, any teaser-slide is present.
  // Problem: $content can be anything, no render array is mandatory
  // (hard-wired HTML may be the case). This also applies to views content.

  // The below comment is inserted by the theme itself, in the node template.
  // @see node.tpl.php
  // @todo review in D8 port (render API).
  if (preg_match(
    '#<!--bilderblock-is-teaser-->#',
    $vars['content'],
    $m
  )) {
    $vars['classes_array'][] = 'bilderblock-has-teasers';
  }
}

/**
 * Override or insert variables into the node template.
 */
function bilderblock_preprocess_node(&$vars) {
  // Provide a single comments link for the teaser view.
  // This link is either the "n comments" or the "add comment"
  // link.
  $v = $vars['content']['links']['comment']['#links'];
  if (
    // Either there are already comments
    ($l =@ $v['comment-comments'])
    ||
    ($l =@ $v['comment-add'])
  ) {
    $vars['comments_link'] = array(
      '#theme' => 'link',
      '#text' => $l['title'],
      '#path' => $l['href'],
      '#options' => array(
        'attributes' => $l['attributes'],
        'fragment' => $l['fragment'],
        'html' => TRUE,
      ),
    );
  }

  // Labels
  $vars['labels'] = theme_get_setting('labels', 'bilderblock');

  // @todo: Provide default image style "bilderblock_slide"?
  // @todo: Provide default image style "bilderblock_exposed_image"?
  if ($nid =@ $vars['node']->nid) {
    if ($imgsource = _bilderblock_image_source($vars['content'], $nid)) {
      $vars['slide'] = $imgsource;
    }
  }
  // Re-format date.
  $vars['date'] = format_date($vars['created'], 'short');
  // @todo: Provide dedicated "slide" date format to configure.

  if (!$fallback = variable_get('bilderblock_fallback', FALSE)) {
    $fallback = bilderblock_fallback_image();
  }
  $vars['fallback'] = $fallback;
}

/**
 * Prepares markup for a static fallback image.
 *
 * @return string
 *   Prepared markup.
 */
function bilderblock_fallback_image() {
  $fallback = theme('image', array(
    // @todo: Make configurable
    'path' => drupal_get_path('theme', 'bilderblock') . '/images/fancy.jpg',
    'title' => t('This post has no exposed image'),
    'attributes' => array(
      'class' => 'teaser-slide-width teaser-slide-height',
    ),
  ));
  variable_set('bilderblock_fallback_image', $fallback);
  return $fallback;
}

/**
 * Implements theme_menu_tree().
 */
function bilderblock_menu_tree($variables) {
  return '<ul class="menu clearfix">' . $variables['tree'] . '</ul>';
}

/**
 * Handles preprocessed dynamic CSS.
 *
 * Checks if the according variable is set, and if not (or if $force is TRUE),
 * preprocesses dynamic CSS with theme settings and updates the variable.
 *
 * @param string $force
 *   Whether to ignore an already existing file.
 * @param object $language
 *   (Optional) Language to use; defaults to the current request language.
 *
 * @return string|bool
 *   The actual dynamic CSS, or FALSE on errors.
 *
 * @see inc/dynamic_styles.inc
 */
function bilderblock_dynamic_css($force = FALSE, $language = NULL) {
  if (!isset($language)) {
    global $language;
  }
  $var_name = 'theme_bilderblock_preprocessed_' . $language->language;
  if (
    ($force)
    ||
    (!$css = variable_get($var_name, FALSE))
  ) {
    // Generate the CSS.
    if ($theme_path = drupal_get_path('theme', 'bilderblock')) {
      $include_path = "${theme_path}/includes/dynamic_styles.inc";
      if (is_readable($include_path)) {
        ob_start();
        include $include_path;
        // No need for strict FALSE check as 0 bytes would indicate an issue, too.
        if ($raw = ob_get_clean()) {
          $css = $raw;
          // Set variable.
          variable_set($var_name, $css);
        }
      }
    }
  }
  return $css;
}

/**
 * Provides the path to the current actual site logo.
 *
 * @param string $stub
 *   (Optional) the theme settings filename stub to process (e.g. "menu_icon").
 *   Defaults to "logo". Intended to allow for looking up logo and menu icon
 *   the same way. @todo review.
 *
 * @return string
 *   The public path to the logo (a URL).
 */
function bilderblock_logo_path($stub = 'logo') {
  $ret = FALSE;
  if (theme_get_setting('toggle_logo', 'bilderblock')) {
    if (theme_get_setting("default_${stub}", 'bilderblock')) {
      $ret = base_path() . drupal_get_path('theme', 'bilderblock') . "/${stub}.png";
    }
    else {
      $ret = file_create_url(theme_get_setting("${stub}_path", 'bilderblock'));
    }
  }
  return $ret;
}

/**
 * Browses a node render array for image sources as of theme settings.
 *
 * @param array $node
 *   A node render array.
 * @param int $nid
 *   The node ID, for internal mapping.
 *
 * @return array|bool
 *   The effective image field source as of the theme settings (render array),
 *   or FALSE if no image source is present.
 */
function _bilderblock_image_source($node, $nid) {
  $static =& drupal_static(__FUNCTION__);
  if (!isset($static['configured_fields'])) {
    $configured_fields = array();
    if ($fields = theme_get_setting('source_fields', 'bilderblock')) {
      // Filter fields by "active" property.
      $fields = array_keys(array_filter((array) $fields, function($val) {
        return $val['active'] == 1;
      }));
      // Sort remaining fields by weight.
      uasort($fields, function($a, $b) {
        if ($a['weight'] > $b['weight']) {
          return 1;
        }
        elseif ($a['weight'] < $b['weight']) {
          return -1;
        }
        else {
          return 0;
        }
      });
      $configured_fields = $fields;
    }
    $static['configured_fields'] = $configured_fields;
  }
  if (!isset($static['available_fields'])) {
    $available_fields = array();
    if ($fields = array_keys(field_info_field_map())) {
      $available_fields = $fields;
    }
    $static['available_fields'] = $available_fields;
  }
  if (!isset($static['sources'][$nid])) {
    $result = FALSE;
    if (!empty($static['configured_fields']) && !empty($static['available_fields'])) {
      foreach ($static['available_fields'] as $fieldname) {
        // Check, if image is set in theme settings.
        if (in_array($fieldname, $static['configured_fields'])) {
          // Check if field still exists.
          if (in_array($fieldname, $static['available_fields'])) {
            // Quit and return as soon as one image is available.
            if ($ret =@ $node[$fieldname]) {
              $result = $ret;
              break; // No further iteration required.
            }
          }
        }
      }
    }
    $static['sources'][$nid] = $result;
  }
  return $static['sources'][$nid];
}
